#!/bin/bash

# Copyright (C) 2019, 2020 Svante Signell

# Written by Svante Signell  <svante.signell@gmail.com>

# This file is part of hurd-cross.

# Hurd-cross is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published
# by the Free Software Foundation; either version 3 or (at your
# option) any later version.

# The hurd-cross is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with the GNU Hurd.  If not, see <http://www.gnu.org/licenses/>.

. ./package-versions.sh
. ./environment-variables.sh

echo ""
echo "File: create-image.sh"

LOOP=/dev/loop0
MAPPER=/dev/mapper/loop0p1
IMG=hurd-cross.img
IMG_SIZE=4000MiB

create_image () {
    print_info "Creating disk image $IMG..."
    fallocate -l $IMG_SIZE $IMG
    losetup $LOOP $IMG
    parted -a optimal -s $LOOP mklabel msdos
    parted -a optimal -s $LOOP -- mkpart primary ext2 1049k -1
    parted -s $LOOP -- set 1 boot on
    kpartx -a -v $LOOP
    sleep 2
    mkfs.ext2 -o hurd -m 1 -v $MAPPER
}

mount_image () {
    print_info "Mounting disk image $IMG..."
    mkdir -p mount
    mount -t ext2 $MAPPER mount
}

# Additional stuff:
# mkdir -p files/etc/pam.d
# cp /etc/pam.d/common-auth files/etc/pam.d/common-auth
# FIXME: Create pam-auth-update in Linux-PAM
# file /usr/sbin/pam-auth-update
# /usr/sbin/pam-auth-update: Perl script text executable
# FIXME: Find out how and create /etc/skel files
# Note: To have an empty root password: remove second entry (*) in files/etc/shadow (same for (!) in $USER)

copy_files () {
    print_info "Copying system into mount..."
    cp -pR /tools/* mount/
    mkdir -p mount/var/run
    cp -pR files/etc mount
    cp -p files/runsystem.hurd mount/libexec/
    mkdir -p mount/etc/hurd
    chmod +x mount/libexec/runsystem.hurd
    mkdir -p mount/boot/grub
    cp -p files/boot/grub.cfg mount/boot/grub
    cp -p files/boot/servers.boot mount/boot
    mkdir -p mount/servers/socket
    touch mount/servers/{exec,crash-kill,default-pager,password,socket,startup,proc,auth,symlink}
    mkdir mount/root
    mkdir mount/tmp
    chmod 01777 mount/tmp
    cp -p files/{rc,runsystem} mount/libexec/
    ln -svf /bin/bash mount/bin/sh
    # Crucial: file .../ld: dynamically linked, interpreter /tools/lib/ld.so, ---
    (cd mount; ln -sf . ./tools)
    (cd mount/lib; ln -sf ld.so.1 ld.so)
    (cd mount; ln -sf . ./usr)
    cp -p files/SETUP mount/
    chmod +x mount/SETUP
    # Create a motd message.
    echo "Welcome to the HURD!" > mount/etc/motd
    echo "Cross-compiled from $HOST to $TARGET on `date`" >> mount/etc/motd
    chown -R root.root mount
}

install_grub () {
   print_info "Installing the GRUB on $IMG..."
   grub-install --target=i386-pc --boot-directory=$PWD/mount/boot $LOOP
}

umount_image () {
    print_info "Unmounting mount..."
    umount mount
    kpartx -d $LOOP
    losetup -d $LOOP
    rmdir mount
}

#umount mount >/dev/null 2>&1
#kpartx -d $LOOP >/dev/null 2>&1
#losetup -d $LOOP >/dev/null 2>&1
umount mount
kpartx -d $LOOP
losetup -d $LOOP
rm -f $IMG
create_image
mount_image
copy_files
install_grub
umount_image
chown $USER.$USER $IMG
print_info "Disk image available on $IMG"
print_info "Run 'qemu-system-x86_64 $IMG' to enjoy the Hurd!"
exit 0
